@@ -51,13 +51,14 @@ Box_Reports::~Box_Reports()
 {
 }
 
-void Box_Reports::fill_row(const Gtk::TreeModel::iterator& iter, const Report& report)
+void Box_Reports::fill_row(const Gtk::TreeModel::iterator& iter, const sharedptr<const Report>& report)
 {
   if(iter)
   {
-    m_AddDel.set_value_key(iter, report.m_name);
-    m_AddDel.set_value(iter, m_colReportName, report.m_name);
-    m_AddDel.set_value(iter, m_colTitle, report.m_title);
+    const Glib::ustring report_name = report->get_name();
+    m_AddDel.set_value_key(iter, report_name);
+    m_AddDel.set_value(iter, m_colReportName, report_name);
+    m_AddDel.set_value(iter, m_colTitle, report->get_title());
   }
 }
 
@@ -84,6 +85,9 @@ bool Box_Reports::fill_from_database()
   const bool editable = developer_mode;
   const bool visible_extras = developer_mode;
   m_colReportName = m_AddDel.add_column(_("Name"), AddDelColumnInfo::STYLE_Text, editable, visible_extras);
+  m_AddDel.prevent_duplicates(m_colReportName); //Don't allow a relationship to be added twice.
+  m_AddDel.set_prevent_duplicates_warning(_("This report already exists. Please choose a different report name"));
+
   m_colTitle = m_AddDel.add_column(_("Title"), AddDelColumnInfo::STYLE_Text, editable, true);
 
   //_("Server: ") +  m_strServerName + ", " + 
@@ -94,14 +98,13 @@ bool Box_Reports::fill_from_database()
   Document_Glom* document = get_document();
   if(document)
   {
-    listTableReports = document->get_report_names(m_strTableName);
+    listTableReports = document->get_report_names(m_table_name);
     for(Document_Glom::type_listReports::const_iterator iter = listTableReports.begin(); iter != listTableReports.end(); ++iter)
     {
-      Report report;
-      bool found = document->get_report(m_strTableName, *iter, report);
-      if(found)
+      sharedptr<Report> report = document->get_report(m_table_name, *iter);
+      if(report)
       {
-        Gtk::TreeModel::iterator row = m_AddDel.add_item(report.m_name);
+        Gtk::TreeModel::iterator row = m_AddDel.add_item(report->get_name());
         fill_row(row, report);
       }
     }
@@ -121,10 +124,16 @@ bool Box_Reports::fill_from_database()
 
 void Box_Reports::on_adddel_Add(const Gtk::TreeModel::iterator& row)
 {
-  Report report;
-  report.m_name = m_AddDel.get_value(row, m_colReportName);
+  sharedptr<Report> report = sharedptr<Report>::create();
 
-  get_document()->set_report(m_strTableName, report);
+  const Glib::ustring report_name = m_AddDel.get_value(row, m_colReportName);
+  if(!report_name.empty())
+  {
+    report->set_name(report_name);
+    m_AddDel.set_value_key(row, report_name);
+
+    get_document()->set_report(m_table_name, report);
+  }
 }
 
 void Box_Reports::on_adddel_Delete(const Gtk::TreeModel::iterator& rowStart, const Gtk::TreeModel::iterator& /* TODO: rowEnd */)
@@ -132,7 +141,7 @@ void Box_Reports::on_adddel_Delete(const Gtk::TreeModel::iterator& rowStart, con
   const Glib::ustring name = m_AddDel.get_value_key(rowStart);
   if(!name.empty())
   {
-    get_document()->remove_report(m_strTableName, name);
+    get_document()->remove_report(m_table_name, name);
     m_AddDel.remove_item_by_key(name);
   }
 }
@@ -156,7 +165,7 @@ void Box_Reports::save_to_document()
   if(get_userlevel() == AppState::USERLEVEL_DEVELOPER)
   {
     //Add any reports that are not in the document:
-    Document_Glom::type_listReports listReports = get_document()->get_report_names(m_strTableName);
+    Document_Glom::type_listReports listReports = get_document()->get_report_names(m_table_name);
 
     bool modified = false;
     for(Gtk::TreeModel::iterator iter = m_AddDel.get_model()->children().begin(); iter != m_AddDel.get_model()->children().end(); ++iter)
@@ -165,12 +174,12 @@ void Box_Reports::save_to_document()
 
       if(!report_name.empty() && std::find(listReports.begin(), listReports.end(), report_name) == listReports.end())
       {
-        Report report;
-        report.m_name = report_name;
+        sharedptr<Report> report(new Report());
+        report->set_name(report_name);
 
-        report.m_title  = m_AddDel.get_value(iter, m_colTitle);
+        report->set_title( m_AddDel.get_value(iter, m_colTitle) ); //TODO: Translations: Store the original in the TreeView.
 
-        get_document()->set_report(m_strTableName, report);
+        get_document()->set_report(m_table_name, report);
         modified = true;
      }
     }
@@ -187,15 +196,15 @@ void Box_Reports::on_adddel_changed(const Gtk::TreeModel::iterator& row, guint c
     const Glib::ustring report_name = m_AddDel.get_value_key(row);
     Document_Glom* document = get_document();
 
-    Report report;
-    bool found = document->get_report(m_strTableName, report_name, report);
-    if(found)
+    sharedptr<Report> report = document->get_report(m_table_name, report_name);
+    if(report)
     {
       if(column == m_colTitle)
       {
-        report.m_title = m_AddDel.get_value(row, m_colTitle);
-        document->set_report(m_strTableName, report);
-      } 
+        report->set_title( m_AddDel.get_value(row, m_colTitle) );
+        //TODO: Unnecessary:
+        document->set_report(m_table_name, report);
+      }
       else if(column == m_colReportName)
       {
         const Glib::ustring report_name_new = m_AddDel.get_value(row, m_colReportName);
@@ -211,10 +220,10 @@ void Box_Reports::on_adddel_changed(const Gtk::TreeModel::iterator& row, guint c
           {
             m_AddDel.set_value_key(row, report_name_new);
 
-            document->remove_report(m_strTableName, report_name);
+            document->remove_report(m_table_name, report_name);
 
-            report.m_name = report_name_new;
-            document->set_report(m_strTableName, report);
+            report->set_name(report_name_new);
+            document->set_report(m_table_name, report);
           }
         }
       }
