@@ -33,8 +33,10 @@ LayoutWidgetBase::LayoutWidgetBase()
   m_refContextLayoutProperties =  Gtk::Action::create("ContextLayoutProperties", _("Field Layout Properties"));
   m_refContextAddField =  Gtk::Action::create("ContextAddField", _("Add Field"));
   m_refContextAddRelatedRecords =  Gtk::Action::create("ContextAddRelatedRecords", _("Add Related Records"));
+  m_refContextAddNotebook =  Gtk::Action::create("ContextAddNotebook", _("Add Notebook"));
   m_refContextAddGroup =  Gtk::Action::create("ContextAddGroup", _("Add Group"));
-
+  m_refContextAddButton =  Gtk::Action::create("ContextAddButton", _("Add Button"));
+  m_refContextAddText =  Gtk::Action::create("ContextAddText", _("Add Text"));
 }
 
 LayoutWidgetBase::~LayoutWidgetBase()
@@ -66,13 +68,22 @@ void LayoutWidgetBase::setup_menu()
     sigc::mem_fun(*this, &LayoutWidgetBase::on_menupopup_activate_layout_properties) );
 
   m_refActionGroup->add(m_refContextAddField,
-    sigc::bind( sigc::mem_fun(*this, &LayoutWidgetBase::on_menupopup_add_item), TreeStore_Layout::TYPE_FIELD ) );
+    sigc::bind( sigc::mem_fun(*this, &LayoutWidgetBase::on_menupopup_add_item), TYPE_FIELD ) );
 
   m_refActionGroup->add(m_refContextAddRelatedRecords,
-    sigc::bind( sigc::mem_fun(*this, &LayoutWidgetBase::on_menupopup_add_item), TreeStore_Layout::TYPE_PORTAL ) );
+    sigc::bind( sigc::mem_fun(*this, &LayoutWidgetBase::on_menupopup_add_item), TYPE_PORTAL ) );
 
   m_refActionGroup->add(m_refContextAddGroup,
-    sigc::bind( sigc::mem_fun(*this, &LayoutWidgetBase::on_menupopup_add_item), TreeStore_Layout::TYPE_GROUP ) );
+    sigc::bind( sigc::mem_fun(*this, &LayoutWidgetBase::on_menupopup_add_item), TYPE_GROUP ) );
+
+  m_refActionGroup->add(m_refContextAddNotebook,
+    sigc::bind( sigc::mem_fun(*this, &LayoutWidgetBase::on_menupopup_add_item), TYPE_NOTEBOOK ) );
+
+  m_refActionGroup->add(m_refContextAddButton,
+    sigc::bind( sigc::mem_fun(*this, &LayoutWidgetBase::on_menupopup_add_item), TYPE_BUTTON ) );
+
+  m_refActionGroup->add(m_refContextAddText,
+    sigc::bind( sigc::mem_fun(*this, &LayoutWidgetBase::on_menupopup_add_item), TYPE_TEXT ) );
 
   //TODO: This does not work until this widget is in a container in the window:s
   App_Glom* pApp = get_application();
@@ -82,7 +93,10 @@ void LayoutWidgetBase::setup_menu()
     pApp->add_developer_action(m_refContextLayoutProperties); //So that it can be disabled when not in developer mode.
     pApp->add_developer_action(m_refContextAddField);
     pApp->add_developer_action(m_refContextAddRelatedRecords);
+    pApp->add_developer_action(m_refContextAddNotebook);
     pApp->add_developer_action(m_refContextAddGroup);
+    pApp->add_developer_action(m_refContextAddButton);
+    pApp->add_developer_action(m_refContextAddText);
 
     pApp->update_userlevel_ui(); //Update our action's sensitivity. 
   }
@@ -102,7 +116,10 @@ void LayoutWidgetBase::setup_menu()
         "    <menuitem action='ContextLayoutProperties'/>"
         "    <menuitem action='ContextAddField'/>"
         "    <menuitem action='ContextAddRelatedRecords'/>"
+        "    <menuitem action='ContextAddNotebook'/>"
         "    <menuitem action='ContextAddGroup'/>"
+        "    <menuitem action='ContextAddButton'/>"
+        "    <menuitem action='ContextAddText'/>"
         "  </popup>"
         "</ui>";
 
@@ -123,7 +140,7 @@ void LayoutWidgetBase::setup_menu()
     m_refContextLayout->set_sensitive(pApp->get_userlevel() == AppState::USERLEVEL_DEVELOPER);
 }
 
-void LayoutWidgetBase::on_menupopup_add_item(TreeStore_Layout::enumType item)
+void LayoutWidgetBase::on_menupopup_add_item(enumType item)
 {
   signal_layout_item_added().emit(item);
 }
